@@ -11,7 +11,7 @@ use Sys::Hostname qw( hostname );
 use AE;
 
 # ABSTRACT: Parallel SSH client
-our $VERSION = '1.00'; # VERSION
+our $VERSION = '1.01'; # VERSION
 
 
 sub main
@@ -39,6 +39,7 @@ sub new
     fat        => 0,
     max        => 0,
     count      => 0,
+    files      => [],
   }, $class;
   
   my @argv = @_;
@@ -56,6 +57,7 @@ sub new
     'config=s' => \$config_name,
     'fat'      => \$self->{fat},
     'max=s'    => \$self->{max},
+    'file=s'   => $self->{files},
     'help|h'   => sub { pod2usage({ -verbose => 2}) },
     'version'  => sub {
       say STDERR 'App::clad version ', ($App::clad::VERSION // 'dev');
@@ -64,7 +66,7 @@ sub new
   ) || pod2usage(1);
   
   $self->{config} = Clustericious::Config->new($config_name);
-  
+
   return $self if $self->server;
   
   # make sure there is at least one cluster specified
@@ -97,6 +99,14 @@ sub new
     }
   }
   
+  if($self->config->script(default => {})->{$self->command->[0]})
+  {
+    my $name = shift @{ $self->command };
+    unshift @{ $self->command }, '$SCRIPT1';
+    my $content = $self->config->script(default => {})->{$name};
+    $self->{script} = [ $name => $content ];
+  }
+  
   my $ok = 1;
   
   foreach my $cluster (map { "$_" } $self->clusters)
@@ -109,6 +119,13 @@ sub new
     }
   }
   
+  foreach my $file ($self->files)
+  {
+    next if -r $file;
+    say STDERR "unable to find $file";
+    $ok = 0;
+  }
+  
   exit 2 unless $ok;
   
   $self;
@@ -124,6 +141,8 @@ sub server         { shift->{server}        }
 sub verbose        { shift->{verbose}       }
 sub serial         { shift->{serial}        }
 sub max            { shift->{max}           }
+sub files          { @{ shift->{files} }    }
+sub script         { @{ shift->{script} // [] } }
 sub ssh_command    { shift->config->ssh_command(    default => 'ssh' ) }
 sub ssh_options    { shift->config->ssh_options(    default => [ -o => 'StrictHostKeyChecking=no', 
                                                                  -o => 'BatchMode=yes',
@@ -279,8 +298,9 @@ use AE;
 use AnyEvent::Open3::Simple 0.76;
 use YAML::XS qw( Dump );
 use JSON::MaybeXS qw( encode_json );
+use File::Basename qw( basename );
 
-our $VERSION = '1.00'; # VERSION
+our $VERSION = '1.01'; # VERSION
 
 sub new
 {
@@ -326,6 +346,8 @@ sub new
     },
   );
   
+  # can/should we generate this payload once, instead
+  # of for each host?
   my $payload = {
     env     => $args{env},
     command => $clad->command,
@@ -333,6 +355,36 @@ sub new
     version => $App::clad::VERSION // 'dev',
   };
   
+  if($self->clad->files)
+  {
+    $payload->{require} = '1.01';
+    
+    foreach my $filename ($self->clad->files)
+    {
+      my %h;
+      open my $fh, '<', $filename;
+      binmode $fh;
+      $h{content} = do { local $/; <$fh> };
+      close $fh;
+      $h{name} = basename $filename;
+      $h{mode} = (stat "/etc/passwd")[2] & 0777;
+      push @{ $payload->{files} }, \%h;
+    }
+  }
+  
+  if($self->clad->script)
+  {
+    my($name, $content) = $self->clad->script;
+    $payload->{require} = '1.01';
+    
+    push @{ $payload->{files} }, {
+      name    => $name,
+      content => $content,
+      mode    => '0700',
+      env     => 'SCRIPT1',
+    };
+  }
+  
   if($self->clad->fat)
   {
     # Perl on the remote end may not have YAML
@@ -406,7 +458,7 @@ App::clad - Parallel SSH client
 
 =head1 VERSION
 
-version 1.00
+version 1.01
 
 =head1 SYNOPSIS
 
