@@ -3,9 +3,11 @@ package Clustericious::Admin::Server;
 use strict;
 use warnings;
 use Sys::Hostname qw( hostname );
+use File::Temp qw( tempdir );
+use File::Spec;
 
 # ABSTRACT: Parallel SSH client server side code
-our $VERSION = '1.00'; # VERSION
+our $VERSION = '1.01'; # VERSION
 
 
 # This is the implementation of the clad server.
@@ -92,6 +94,12 @@ sub _server
   #     server should die if requirement isn't met
   #     ignored if set to 'dev'
   #
+  #   files: optional list of hashref
+  #     each hashref has:
+  #       name: the file basename (no directory)
+  #       content: the content of the file
+  #       mode: (optional) octal unix permission mode as a string (ie "0755" or "0644")
+  #       env: (optional) environment variable to use instead of FILEx
 
   if(ref $payload->{command} ne 'ARRAY' || @{ $payload->{command} } == 0)
   {
@@ -111,15 +119,32 @@ sub _server
     return 2;
   }
   
-  if($payload->{require} && defined $App::clad::VERSION)
+  if($payload->{require} && defined $Clustericious::Admin::Server::VERSION)
   {
-    if($payload->{require} ne 'dev' && $payload->{require} > $App::clad::VERSION)
+    if($payload->{require} ne 'dev' && $payload->{require} > $Clustericious::Admin::Server::VERSION)
     {
-      print STDERR "Clad Server: client requested version @{[ $payload->{require} ]} but this is only $App::clad::VERSION\n";
+      print STDERR "Clad Server: client requested version @{[ $payload->{require} ]} but this is only $Clustericious::Admin::Server::VERSION\n";
       return 2;
     }
   }
-  
+
+  if($payload->{files})
+  {
+    my $count = 1;
+    foreach my $file (@{ $payload->{files} })
+    {
+      my $path = File::Spec->catfile( tempdir( CLEANUP => 1 ), $file->{name} );
+      open my $fh, '>', $path;
+      binmode $fh;
+      print $fh $file->{content};
+      close $fh;
+      chmod oct($file->{mode}), $path if defined $file->{mode};
+      my $env = $file->{env};
+      $env = "FILE@{[ $count++ ]}" unless defined $env;
+      $ENV{$env} = $path;
+    }
+  }
+
   $ENV{$_} = $payload->{env}->{$_} for keys %{ $payload->{env} };
   
   system @{ $payload->{command} };
@@ -152,7 +177,7 @@ Clustericious::Admin::Server - Parallel SSH client server side code
 
 =head1 VERSION
 
-version 1.00
+version 1.01
 
 =head1 SYNOPSIS
 
